<?php

namespace AltSolution\Admin\Exceptions;

use Throwable;
use Illuminate\Auth\Access\AuthorizationException;
use Illuminate\Auth\AuthenticationException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Illuminate\Contracts\Debug\ExceptionHandler as ExceptionHandlerContract;
use Illuminate\Session\TokenMismatchException;

class Handler implements ExceptionHandlerContract
{
    /**
     * @var ExceptionHandlerContract|null
     */
    private $previous;

    public function __construct(ExceptionHandlerContract $previous = null)
    {
        $this->previous = $previous;
    }

    /**
     * Report or log an exception.
     *
     * @param  \Throwable  $e
     * @return void
     */
    public function report(Throwable $e)
    {
        if ($this->previous !== null) {
            $this->previous->report($e);
        }
        // TODO: log 403, 404 errors
    }

    /**
     * Determine if the exception should be reported.
     *
     * @param  \Throwable  $e
     * @return bool
     */
    public function shouldReport(Throwable $e)
    {
        if ($this->previous !== null) {
            return $this->previous->shouldReport($e);
        }
        return false;
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Throwable  $e
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function render($request, Throwable $e)
    {
        $backendUrl = cms_admin_url();
        if ($request->is($backendUrl, $backendUrl . '/*')) {
            if ($e instanceof HttpException) {
                $statusCode = $e->getStatusCode();
                if ($statusCode === 403) {
                    return $this->unauthorized($request, $e);
                }
                if ($statusCode === 404) {
                    return $this->notfound($request, $e);
                }
                if ($statusCode === 500) {
                    return $this->serverError($request, $e);
                }
            }
            if ($e instanceof AuthorizationException) {
                return $this->unauthorized($request, $e);
            }
            if ($e instanceof AuthenticationException) {
                return $this->unauthenticated($request, $e);
            }
            if ($e instanceof TokenMismatchException) {
                //
            }
        }

        if ($this->previous !== null) {
            return $this->previous->render($request, $e);
        }

        return $this->serverError($request, $e);
    }

    /**
     * Render an exception to the console.
     *
     * @param  \Symfony\Component\Console\Output\OutputInterface  $output
     * @param  \Throwable  $e
     * @return void
     */
    public function renderForConsole($output, Throwable $e)
    {
        if ($this->previous !== null) {
            $this->previous->renderForConsole($output, $e);
        }
    }

    /**
     * Convert an notfound exception into a response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Symfony\Component\HttpKernel\Exception\NotFoundHttpException  $exception
     * @return \Illuminate\Http\Response
     */
    protected function notfound($request, NotFoundHttpException $exception)
    {
        if ($request->expectsJson()) {
            return response()->json(['message' => $exception->getMessage()], 404);
        }

        $layout = app(\AltSolution\Admin\System\AuthLayout::class);
        $layout->setTitle(trans('admin::errors.not_found'));
        return response()->view('admin::errors.404', [], 404);
    }

    /**
     * Convert an authorization exception into a response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Illuminate\Auth\Access\AuthorizationException|\Throwable  $exception
     * @return \Illuminate\Http\Response
     */
    protected function unauthorized($request, $exception)
    {
        if ($request->expectsJson()) {
            return response()->json(['message' => $exception->getMessage()], 403);
        }

        $layout = app(\AltSolution\Admin\System\AuthLayout::class);
        $layout->setTitle(trans('admin::auth.access_is_denied'));
        return response()->view('admin::errors.403', [], 403);
    }

    /**
     * Convert an exception into a response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Throwable  $exception
     * @return \Illuminate\Http\Response
     */
    protected function serverError($request, $exception)
    {
        if ($request->expectsJson()) {
            return response()->json(['message' => $exception->getMessage()], 500);
        }

        $layout = app(\AltSolution\Admin\System\AuthLayout::class);
        $layout->setTitle(trans('admin::errors.server_error'));
        return response()->view('admin::errors.500', [], 500);
    }

    /**
     * Convert an authentication exception into a response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Illuminate\Auth\AuthenticationException  $exception
     * @return \Illuminate\Http\Response
     */
    protected function unauthenticated($request, AuthenticationException $exception)
    {
        if ($request->expectsJson()) {
            return response()->json(['message' => $exception->getMessage()], 401);
        }

        return redirect()->guest(route('admin::login'));
    }
}
