<?php

namespace AltSolution\Admin\Form;

use Illuminate\Support\Arr;
use Traversable;

class Builder implements BuilderInterface
{
    /**
     * @var string
     */
    private $prefix;
    /**
     * @var ComponentInterface[]
     */
    private $components = [];
    /**
     * @var bool
     */
    private $locked = false;
    /**
     * @var mixed
     */
    private $dataSource;

    public function getPrefix()
    {
        return $this->prefix;
    }

    public function setPrefix($prefix)
    {
        $this->prefix = $prefix;

        return $this;
    }

    public function setDataSource($dataSource)
    {
        $this->dataSource = $dataSource;

        return $this;
    }

    public function getDataSource()
    {
        return $this->dataSource;
    }

    public function getData($key)
    {
        if ($this->dataSource === null) {
            return null;
        }
        return Arr::get($this->dataSource, $key);
    }

    public function build()
    {
        $form = new Form();

        foreach ($this->components as $componentName => $componentData) {
            list($componentClass, $componentOptions) = $componentData;
            $name = $this->prefix . $componentName;
            $component = new $componentClass($name, $componentOptions);

            $form->add($component);
        }

        if ($this->dataSource) {
            $form->setDataSource($this->dataSource);
        }

        $this->locked = true;

        return $form;
    }

    public function add($name, $component, array $options = [])
    {
        if ($this->locked) {
            throw new \Exception('Builder locked after getting form');
        }
        if (isset($this->components[$name])) {
            throw new \Exception('Component already exists: ' . $name);
        }
        $this->components[$name] = [$component, $options];

        return $this;
    }

    public function remove($name)
    {
        if ($this->locked) {
            throw new \Exception('Builder locked after getting form');
        }
        unset($this->components[$name]);

        return $this;
    }

    public function addMany($fields)
    {
        if (is_array($fields) || ($fields instanceof Traversable)) {
            foreach ($fields as $name => $options) {
                $type = Arr::get($options, 'type');
                if ($type === null) {
                    throw new \Exception('Field type is undefined: ' . $name);
                }
                unset($options['type']);
                if (is_numeric($name)) {
                    call_user_func([$this, $type], $options);
                } else {
                    call_user_func([$this, $type], $name, $options);
                }
            }
        }

        return $this;
    }

    public function form(array $options = [])
    {
        return $this
            ->add('form_open', Component\FormOpen::class, $options)
            ->add('form_submit', Component\FormSubmit::class, $options)
            ->add('form_close', Component\FormClose::class);
    }

    function csrfToken()
    {
        return $this->add('_token', Field\CsrfToken::class);
    }

    function header($name, array $options = [])
    {
        return $this->add($name, Component\Header::class, $options);
    }

    function hidden($name, array $options = [])
    {
        return $this->add($name, Field\Hidden::class, $options);
    }

    function checkbox($name, array $options = [])
    {
        return $this->add($name, Field\Checkbox::class, $options);
    }

    function confirm($name, array $options = [])
    {
        return $this->add($name, Field\Confirm::class, $options);
    }

    function toggle($name, array $options = [])
    {
        return $this->add($name, Field\Toggle::class, $options);
    }

    function date($name, array $options = [])
    {
        return $this->add($name, Field\Date::class, $options);
    }

    function dateRange($name, array $options = [])
    {
        return $this->add($name, Field\DateRange::class, $options);
    }

    function datetime($name, array $options = [])
    {
        return $this->add($name, Field\DateTime::class, $options);
    }

    function email($name, array $options = [])
    {
        return $this->add($name, Field\Email::class, $options);
    }

    function password($name, array $options = [])
    {
        return $this->add($name, Field\Password::class, $options);
    }

    function select($name, array $options = [])
    {
        return $this->add($name, Field\Select::class, $options);
    }

    function selectModel($name, array $options = [])
    {
        return $this->add($name, Field\SelectModel::class, $options);
    }

    function select2($name, array $options = [])
    {
        return $this->add($name, Field\Select2::class, $options);
    }

    function text($name, array $options = [])
    {
        return $this->add($name, Field\Text::class, $options);
    }

    function number($name, array $options = [])
    {
        return $this->add($name, Field\Number::class, $options);
    }

    function range($name, array $options = [])
    {
        return $this->add($name, Field\Range::class, $options);
    }

    function slug($name, array $options = [])
    {
        return $this->add($name, Field\Slug::class, $options);
    }

    function textarea($name, array $options = [])
    {
        return $this->add($name, Field\Textarea::class, $options);
    }

    function wysiwyg($name, array $options = [])
    {
        return $this->add($name, Field\Wysiwyg::class, $options);
    }

    function image($name, array $options = [])
    {
        return $this->add($name, Field\Image::class, $options);
    }

    function imageUrl($name, array $options = [])
    {
        return $this->add($name, Field\ImageUrl::class, $options);
    }

    function thumbnail($name, array $options = [])
    {
        return $this->add($name, Field\Thumbnail::class, $options);
    }

    function upload($name, array $options = [])
    {
        return $this->add($name, Field\Upload::class, $options);
    }

    function svg($name, array $options = [])
    {
        return $this->add($name, Field\Svg::class, $options);
    }

    function choice($name, array $options = [])
    {
        return $this->add($name, Field\Choice::class, $options);
    }

    function partial($name, array $options = [])
    {
        return $this->add($name, Field\Partial::class, $options);
    }

    function color($name, array $options = [])
    {
        return $this->add($name, Field\Color::class, $options);
    }
    
    function code($name, array $options = [])
    {
        return $this->add($name, Field\Code::class, $options);
    }
}
